
#ifndef _IPV4_ADDRESS_H
#define _IPV4_ADDRESS_H

union Inetv4Address
{
	struct S_un_b
	{
		unsigned char ub[4];
	} un_b;
	struct S_un_w
	{
		unsigned short uw[2];
	} un_w;
	unsigned long addr_l;
};

struct AdapterAddress
{
    Inetv4Address ip;
    Inetv4Address mask;
    Inetv4Address gw;
};

enum InetResult
{
    INET_OK = 0,
    INET_INVALID_PARAMETER,
    INET_INVALID_HOST,
    INET_INVALID_MASK,
    INET_INVALID_GATEWAY,
    INET_INVALID_ADDRESS,
    INET_INVALID_ADDRESS_TYPE,
    INET_MULTICAST_ADDRESS,
    INET_RESERVED_ADDRESS,
    INET_LOOPBACK_ADDRESS,
    INET_NETWORK_HOST_ADDRESS,
    INET_NETWORK_GATEWAY_ADDRESS,
    INET_BROADCAST_HOST_ADDRESS,
    INET_BROADCAST_GATEWAY_ADDRESS,
    INET_ADDRESS_NOT_MATCH
};

class IPv4AddressUtil
{
    private:
        IPv4AddressUtil();
        static bool ValidateMask(Inetv4Address mask);
        static bool IsBroadcastAddress(Inetv4Address inetAddr, Inetv4Address mask);
        static bool IsMulticastAddress(Inetv4Address hostAddr);

    public:
        virtual ~IPv4AddressUtil();

        static InetResult ValidateHostAddress(const Inetv4Address& hostAddr);
        static InetResult ValidateHostAddress(const char* pszHostAddr);
        static InetResult StringToInetAddress(const char* pszIpAddr, Inetv4Address* pInetAddr);
        static InetResult InetAddressToString(const Inetv4Address* pInetAddr, char* pszIpAddr);
        static InetResult ValidateAdapterAddress(const AdapterAddress* pAdptAddr, bool strictType = false);
        static InetResult ValidateAdapterAddress(const char* pszHost, const char* pszMask, const char* pszGateway, bool strictType = false);
};

#endif // _IPV4_ADDRESS_H
